package io.gitlab.jfronny.googlechat.mixin;

import io.gitlab.jfronny.googlechat.GoogleChat;
import io.gitlab.jfronny.googlechat.GoogleChatConfig;
import io.gitlab.jfronny.libjf.translate.api.Language;
import io.gitlab.jfronny.libjf.translate.api.TranslateException;
import io.gitlab.jfronny.libjf.translate.api.TranslateService;
import net.fabricmc.api.EnvType;
import net.fabricmc.loader.api.FabricLoader;
import net.minecraft.network.MessageType;
import net.minecraft.network.PacketByteBuf;
import net.minecraft.network.packet.s2c.play.GameMessageS2CPacket;
import net.minecraft.text.*;
import org.spongepowered.asm.mixin.Final;
import org.spongepowered.asm.mixin.Mixin;
import org.spongepowered.asm.mixin.Mutable;
import org.spongepowered.asm.mixin.Shadow;
import org.spongepowered.asm.mixin.injection.At;
import org.spongepowered.asm.mixin.injection.Inject;
import org.spongepowered.asm.mixin.injection.callback.CallbackInfo;

import java.util.List;
import java.util.UUID;

@Mixin(GameMessageS2CPacket.class)
public class GameMessageS2CPacketMixin {
    @Shadow @Final @Mutable private Text message;
    @Shadow @Final private UUID sender;

    @Inject(at = @At("RETURN"), method = "<init>(Lnet/minecraft/text/Text;Lnet/minecraft/network/MessageType;Ljava/util/UUID;)V")
    private void init(Text message, MessageType type, UUID sender, CallbackInfo ci) {
        googlechat$translate(GoogleChat.TRANSLATE_SERVICE);
    }

    @Inject(at = @At("RETURN"), method = "<init>(Lnet/minecraft/network/PacketByteBuf;)V")
    private void init(PacketByteBuf buf, CallbackInfo ci) {
        googlechat$translate(GoogleChat.TRANSLATE_SERVICE);
    }

    private <T extends Language> void googlechat$translate(TranslateService<T> ts) {
        if (!GoogleChatConfig.enabled) return;
        if (FabricLoader.getInstance().getEnvironmentType() == EnvType.CLIENT && GoogleChat.isSelf(sender)) {
            return;
        }
        T client = ts.parseLang(GoogleChatConfig.clientLanguage);
        if (client == ts.parseLang("auto")) return;
        StringBuilder sb = new StringBuilder();
        message.asOrderedText().accept((index, style, codePoint) -> {
            sb.append((char)codePoint);
            return true;
        });
        try {
            LiteralText translatedText = new LiteralText(ts.translate(sb.toString(), ts.parseLang(GoogleChatConfig.serverLanguage), client));
            if (GoogleChatConfig.translationTooltip)
                message = googlechat$concat(message.getWithStyle(Style.EMPTY.withHoverEvent(new HoverEvent(HoverEvent.Action.SHOW_TEXT, new LiteralText("Translated: ").append(translatedText)))));
            else
                message = translatedText.setStyle(Style.EMPTY.withHoverEvent(new HoverEvent(HoverEvent.Action.SHOW_TEXT, new LiteralText("Original: ").append(message))));
        } catch (TranslateException e) {
            GoogleChat.LOGGER.error("Could not translate received message", e);
        }
    }

    private Text googlechat$concat(List<Text> texts) {
        MutableText res = null;
        for (Text text : texts) {
            res = res == null ? (MutableText) text : res.append(text);
        }
        return res;
    }
}
